@@ -10,20 +10,16 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
 #include "impl.h"
 #include <ctype.h>
 
-static __pmLogPort *logport = NULL;
+static __pmLogPort *logport;
 					/* array of all known pmlogger ports */
-static int	nlogports = 0;		/* no. of elements used in logports array */
-static int	szlogport = 0;		/* size of logport array */
+static int	nlogports;		/* no. of elements used in logports array */
+static int	szlogport;		/* size of logport array */
 
 /* Make sure the logports array is large enough to hold newsize entries.  Free
  * any currently allocated names and zero the first newsize entries.
@@ -66,12 +62,12 @@ static int
 is_portfile(const_dirent *dep)
 {
     char	*endp;
-    int		pid;
+    pid_t	pid;
 
-    pid = (int)strtol(dep->d_name, &endp, 10);
-    if (pid > 1)
+    pid = (pid_t)strtol(dep->d_name, &endp, 10);
+    if (pid > (pid_t)1)
 	return __pmProcessExists(pid);
-    return strcmp(dep->d_name, PM_LOG_PRIMARY_LINK) == 0;
+    return strcmp(dep->d_name, "primary") == 0;
 }
 
 /* The following function is used for selecting particular port files rather
@@ -96,11 +92,11 @@ is_match(const_dirent *dep)
 int
 __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 {
-    static char		*dir = NULL;
+    static char		dir[MAXPATHLEN];
     static int		lendir;
     int			i, j, n;
     int			nf;		/* number of port files found */
-    struct dirent	**files;	/* array of port file dirents */
+    struct dirent	**files = NULL;	/* array of port file dirents */
     char		*p;
     int			len;
     static char		*namebuf = NULL;
@@ -113,10 +109,9 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
     if (result == NULL)
 	return -EINVAL;
 
-    if (dir == NULL) {
-	dir = PM_LOG_PORT_DIR;
-	lendir = (int)strlen(dir);
-    }
+    if (lendir == 0)
+	lendir = snprintf(dir, sizeof(dir), "%s%cpmlogger",
+		pmGetConfig("PCP_TMP_DIR"), __pmPathSeparator());
 
     /* Set up the appropriate function to select files from the control port
      * directory.  Anticipate that this will usually be an exact match for
@@ -125,7 +120,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
     scanfn = is_match;
     switch (pid) {
 	case PM_LOG_PRIMARY_PID:	/* primary logger control (single) */
-	    strcpy(match, PM_LOG_PRIMARY_LINK);
+	    strcpy(match, "primary");
 	    break;
 
 	case PM_LOG_ALL_PIDS:		/* find all ports */
@@ -133,7 +128,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	    break;
 
 	default:			/* a specific pid (single) */
-	    if (!__pmProcessExists(pid)) {
+	    if (!__pmProcessExists((pid_t)pid)) {
 #ifdef PCP_DEBUG
 		if (pmDebug & DBG_TRACE_LOG) {
 		    fprintf(stderr, "__pmLogFindLocalPorts() -> 0, "
@@ -147,13 +142,20 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	    break;
     }
 
-    if ((nf = scandir(dir, &files, scanfn, alphasort)) == -1) {
-	pmprintf("__pmLogFindLocalPorts: scandir: %s\n", strerror(errno));
+    nf = scandir(dir, &files, scanfn, alphasort);
+    if (nf == -1 && oserror() == ENOENT)
+	nf = 0;
+    else if (nf == -1) {
+	pmprintf("__pmLogFindLocalPorts: scandir: %s\n", osstrerror());
 	pmflush();
-	return -errno;
+	return -oserror();
+    }
+    if (resize_logports(nf) < 0) {
+	for (i=0; i < nf; i++)
+	    free(files[i]);
+	free(files);
+	return -oserror();
     }
-    if (resize_logports(nf) < 0)
-	return -errno;
     if (nf == 0) {
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_LOG) {
@@ -162,6 +164,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	}
 #endif
 	*result = NULL;
+	free(files);
 	return 0;
     }
 
@@ -170,14 +173,17 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
     for (i = 1; i < nf; i++)
 	if ((j = (int)strlen(files[i]->d_name)) > len)
 	    len = j;
-    /* +1 for trailing /, +1 for null termination */
+    /* +1 for trailing path separator, +1 for null termination */
     len += lendir + 2;
     if (len > sznamebuf) {
 	if (namebuf != NULL)
 	    free(namebuf);
 	if ((namebuf = (char *)malloc(len)) == NULL) {
 	    __pmNoMem("__pmLogFindLocalPorts.namebuf", len, PM_RECOV_ERR);
-	    return -errno;
+	    for (i=0; i < nf; i++)
+		free(files[i]);
+	    free(files);
+	    return -oserror();
 	}
 	sznamebuf = len;
     }
@@ -187,7 +193,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
      */
     strcpy(namebuf, dir);
     p = namebuf + lendir;
-    *p++ = '/';
+    *p++ = __pmPathSeparator();
 
     /* open the file, try to read the port number and add the port to the
      * logport array if successful.
@@ -200,7 +206,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	strcpy(p, fname);
 	if ((pfile = fopen(namebuf, "r")) == NULL) {
 	    pmprintf("__pmLogFindLocalPorts: pmlogger port file %s: %s\n",
-		    namebuf, strerror(errno));
+		    namebuf, osstrerror());
 	    free(files[i]);
 	    pmflush();
 	    continue;
@@ -213,7 +219,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	    }
 	    else
 		pmprintf("__pmLogFindLocalPorts: pmlogger port file %s: %s\n",
-			namebuf, strerror(errno));
+			namebuf, osstrerror());
 	    err = 1;
 	}
 	else {
@@ -228,7 +234,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	    else {
 		lpp->pid = (int)strtol(fname, &endp, 10);
 		if (*endp != '\0') {
-		    if (strcmp(fname, PM_LOG_PRIMARY_LINK) == 0)
+		    if (strcmp(fname, "primary") == 0)
 			lpp->pid = PM_LOG_PRIMARY_PORT;
 		    else {
 			pmprintf("__pmLogFindLocalPorts: unrecognised pmlogger port file %s\n",
@@ -291,7 +297,7 @@ __pmLogFindLocalPorts(int pid, __pmLogPort **result)
 	*result = NULL;
 	for (j = i; j < nf; j++)
 	    free(files[j]);
-	n = -errno;
+	n = -oserror();
     }
     free(files);
     return n;
@@ -313,7 +319,7 @@ __pmIsLocalhost(const char *hostname)
 	struct hostent * he;
 
 	if (gethostname(lhost, MAXHOSTNAMELEN) < 0)
-	   return -errno;
+	   return -oserror();
 
         if ((he = gethostbyname(lhost)) != NULL ) {
 	    int i;
@@ -392,7 +398,7 @@ __pmLogFindPort(const char *host, int pid, __pmLogPort **lpp)
     j = 0;
     if (numval) {
 	if (resize_logports(findone ? 1 : numval) < 0) {
-	    sts = -errno;
+	    sts = -oserror();
 	    goto resErr;
 	}
 	/* scan the pmResult, copying matching pid(s) to logport */
