@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <ctype.h>
@@ -34,9 +30,10 @@ __pmSendLogRequest(int fd, int type)
     notify_t	*pp;
 
     if ((pp = (notify_t *)__pmFindPDUBuf(sizeof(notify_t))) == NULL)
-	return -errno;
+	return -oserror();
     pp->hdr.len = sizeof(notify_t);
     pp->hdr.type = PDU_LOG_REQUEST;
+    pp->hdr.from = FROM_ANON;		/* context does not matter here */
     pp->type = htonl(type);
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_PDU) {
