@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <ctype.h>
@@ -35,9 +31,10 @@ __pmSendLogStatus(int fd, __pmLoggerStatus *status)
     logstatus_t	*pp;
 
     if ((pp = (logstatus_t *)__pmFindPDUBuf(sizeof(logstatus_t))) == NULL)
-	return -errno;
+	return -oserror();
     pp->hdr.len = sizeof(logstatus_t);
     pp->hdr.type = PDU_LOG_STATUS;
+    pp->hdr.from = FROM_ANON;		/* context does not matter here */
     memcpy(&pp->status, status, sizeof(__pmLoggerStatus));
 
     /* Conditional convertion from host to network byteorder HAVE to be
