@@ -41,7 +41,7 @@ static pmdaInterface		dispatch;
 static int			_isDSO = 1;	/* =0 I am a daemon */
 
 mach_port_t		mach_host = 0;
-unsigned int		mach_page_size = 0;
+vm_size_t		mach_page_size = 0;
 unsigned int		mach_page_shift = 0;
 
 unsigned int		mach_hertz = 0;
@@ -150,20 +150,11 @@ enum {
     NUM_CLUSTERS		/* total number of clusters */
 };
 
-#ifdef HAVE_NQNFS_LEASES
-#define NQNFS_LEASE(x) x
-#define PM_TYPE_NQNFS_LEASE PM_TYPE_32
-#else
-static int lease;
-#define NQNFS_LEASE(x) &lease
-#define PM_TYPE_NQNFS_LEASE PM_TYPE_NOSUPPORT
-#endif
-
 static pmdaMetric metrictab[] = {
 
 /* hinv.pagesize */
   { &mach_page_size, 
-    { PMDA_PMID(CLUSTER_INIT,0), PM_TYPE_U32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_INIT,0), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_DISCRETE, PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) }, },
 /* kernel.all.hz */
   { &mach_hertz,
@@ -336,19 +327,19 @@ static pmdaMetric metrictab[] = {
 
 /* kernel.all.cpu.user */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPULOAD,42), PM_TYPE_U32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_CPULOAD,42), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.all.cpu.nice */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPULOAD,43), PM_TYPE_U32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_CPULOAD,43), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.all.cpu.sys */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPULOAD,44), PM_TYPE_U32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_CPULOAD,44), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.all.cpu.idle */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPULOAD,45), PM_TYPE_U32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_CPULOAD,45), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 
 /* hinv.ndisk */
@@ -458,19 +449,19 @@ static pmdaMetric metrictab[] = {
       PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* kernel.percpu.cpu.user */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPU,72), PM_TYPE_U32, CPU_INDOM,
+    { PMDA_PMID(CLUSTER_CPU,72), PM_TYPE_U64, CPU_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.percpu.cpu.nice */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPU,73), PM_TYPE_U32, CPU_INDOM,
+    { PMDA_PMID(CLUSTER_CPU,73), PM_TYPE_U64, CPU_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.percpu.cpu.sys */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPU,74), PM_TYPE_U32, CPU_INDOM,
+    { PMDA_PMID(CLUSTER_CPU,74), PM_TYPE_U64, CPU_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 /* kernel.percpu.cpu.idle */
   { NULL,
-    { PMDA_PMID(CLUSTER_CPU,75), PM_TYPE_U32, CPU_INDOM,
+    { PMDA_PMID(CLUSTER_CPU,75), PM_TYPE_U64, CPU_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) }, },
 
 /* kernel.all.uptime */
@@ -663,17 +654,17 @@ static pmdaMetric metrictab[] = {
   { &mach_nfs.srvcache_misses,
     { PMDA_PMID(CLUSTER_NFS,122), PM_TYPE_32, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
-/* rpc.server.nqnfs.leases */
-  { NQNFS_LEASE(&mach_nfs.srvnqnfs_leases),
-    { PMDA_PMID(CLUSTER_NFS,123), PM_TYPE_NQNFS_LEASE, PM_INDOM_NULL,
+/* rpc.server.nqnfs.leases -- deprecated */
+  { NULL,
+    { PMDA_PMID(CLUSTER_NFS,123), PM_TYPE_NOSUPPORT, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
-/* rpc.server.nqnfs.maxleases */
-  { NQNFS_LEASE(&mach_nfs.srvnqnfs_maxleases),
-    { PMDA_PMID(CLUSTER_NFS,124), PM_TYPE_NQNFS_LEASE, PM_INDOM_NULL,
+/* rpc.server.nqnfs.maxleases -- deprecated */
+  { NULL,
+    { PMDA_PMID(CLUSTER_NFS,124), PM_TYPE_NOSUPPORT, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
-/* rpc.server.nqnfs.getleases */
-  { NQNFS_LEASE(&mach_nfs.srvnqnfs_getleases),
-    { PMDA_PMID(CLUSTER_NFS,125), PM_TYPE_NQNFS_LEASE, PM_INDOM_NULL,
+/* rpc.server.nqnfs.getleases -- deprecated */
+  { NULL,
+    { PMDA_PMID(CLUSTER_NFS,125), PM_TYPE_NOSUPPORT, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.vopwrites */
   { &mach_nfs.srvvop_writes,
@@ -688,6 +679,11 @@ static pmdaMetric metrictab[] = {
     { PMDA_PMID(CLUSTER_NFS,128), PM_TYPE_32, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 
+/* filesys.maxfiles */
+  { NULL,
+     { PMDA_PMID(CLUSTER_FILESYS,129), PM_TYPE_U32, FILESYS_INDOM,
+       PM_SEM_INSTANT, PMDA_PMUNITS(0,0,0,0,0,0) }, },
+
 };
 
 static void
@@ -742,19 +738,19 @@ fetch_cpuload(unsigned int item, pmAtomValue *atom)
 	return mach_cpuload_error;
     switch (item) {
     case 42: /* kernel.all.cpu.user */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 		mach_cpuload.cpu_ticks[CPU_STATE_USER] / mach_hertz;
         return 1;
     case 43: /* kernel.all.cpu.nice */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 		mach_cpuload.cpu_ticks[CPU_STATE_NICE] / mach_hertz;
         return 1;
     case 44: /* kernel.all.cpu.sys */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 		mach_cpuload.cpu_ticks[CPU_STATE_SYSTEM] / mach_hertz;
         return 1;
     case 45: /* kernel.all.cpu.idle */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 		mach_cpuload.cpu_ticks[CPU_STATE_IDLE] / mach_hertz;
         return 1;
     }
@@ -787,7 +783,7 @@ fetch_vmstat(unsigned int item, unsigned int inst, pmAtomValue *atom)
 	atom->ull = page_count_to_kb(mach_vmstat.inactive_count);
 	return 1;
     case 19: /* mem.util.wired */
-	atom->ull = page_count_to_kb(mach_vmstat.free_count);
+	atom->ull = page_count_to_kb(mach_vmstat.wire_count);
 	return 1;
     case 20: /* mem.util.active */
 	atom->ull = page_count_to_kb(mach_vmstat.active_count);
@@ -852,9 +848,12 @@ fetch_filesys(unsigned int item, unsigned int inst, pmAtomValue *atom)
 	ull = (__uint64_t)mach_fs[inst].f_bfree;
 	atom->ull = ull * mach_fs[inst].f_bsize >> 10;
 	return 1;
-    case 35: /* filesys.usedfiles */
+    case 129: /* filesys.maxfiles */
 	atom->ul = mach_fs[inst].f_files;
 	return 1;
+    case 35: /* filesys.usedfiles */
+	atom->ul = mach_fs[inst].f_files - mach_fs[inst].f_ffree;
+	return 1;
     case 36: /* filesys.freefiles */
 	atom->ul = mach_fs[inst].f_ffree;
 	return 1;
@@ -991,19 +990,19 @@ fetch_cpu(unsigned int item, unsigned int inst, pmAtomValue *atom)
 	return PM_ERR_INST;
     switch (item) {
     case 72: /* kernel.percpu.cpu.user */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 			mach_cpu[inst].cpu_ticks[CPU_STATE_USER] / mach_hertz;
 	return 1;
     case 73: /* kernel.percpu.cpu.nice */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 			mach_cpu[inst].cpu_ticks[CPU_STATE_NICE] / mach_hertz;
 	return 1;
     case 74: /* kernel.percpu.cpu.sys */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 			mach_cpu[inst].cpu_ticks[CPU_STATE_SYSTEM] / mach_hertz;
 	return 1;
     case 75: /* kernel.percpu.cpu.idle */
-	atom->ul = LOAD_SCALE * (double)
+	atom->ull = LOAD_SCALE * (double)
 			mach_cpu[inst].cpu_ticks[CPU_STATE_IDLE] / mach_hertz;
 	return 1;
     }
@@ -1103,6 +1102,10 @@ fetch_nfs(unsigned int item, unsigned int inst, pmAtomValue *atom)
 	    return PM_ERR_INST;
 	atom->l = mach_nfs.srvrpccnt[inst];
 	return 1;
+    case 123:	/* rpc.server.nqnfs.leases    -- deprecated */
+    case 124:	/* rpc.server.nqnfs.maxleases -- deprecated */
+    case 125:	/* rpc.server.nqnfs.getleases -- deprecated */
+	return PM_ERR_APPVERSION;
     }
     return PM_ERR_PMID;
 }
@@ -1128,13 +1131,10 @@ darwin_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	case PM_TYPE_FLOAT:	atom->f = *(float *)mdesc->m_user; break;
 	case PM_TYPE_DOUBLE:	atom->d = *(double *)mdesc->m_user; break;
 	case PM_TYPE_STRING:	atom->cp = (char *)mdesc->m_user; break;
-	case PM_TYPE_AGGREGATE_STATIC:
-	case PM_TYPE_AGGREGATE:	atom->vp = (void *)mdesc->m_user; break;
 	case PM_TYPE_NOSUPPORT: return 0;
-	case PM_TYPE_UNKNOWN:
 	default:		fprintf(stderr,
-			"Error in fetchCallBack: unknown metric type %d\n",
-					mdesc->m_desc.type);
+			"Error in fetchCallBack: unsupported metric type %s\n",
+					pmTypeStr(mdesc->m_desc.type));
 				return 0;
 	}
 	return 1;
@@ -1188,8 +1188,10 @@ void
 darwin_init(pmdaInterface *dp)
 {
     if (_isDSO) {
+	int sep = __pmPathSeparator();
 	char helppath[MAXPATHLEN];
-	sprintf(helppath, "%s/pmdas/darwin/help", pmGetConfig("PCP_VAR_DIR"));
+	sprintf(helppath, "%s%c" "darwin" "%c" "help",
+		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
 	pmdaDSO(dp, PMDA_INTERFACE_3, "darwin DSO", helppath);
     }
 
@@ -1202,9 +1204,10 @@ darwin_init(pmdaInterface *dp)
 
     pmdaInit(dp, indomtab, sizeof(indomtab)/sizeof(indomtab[0]),
 		metrictab, sizeof(metrictab)/sizeof(metrictab[0]));
-    /* Make sure we have a direct metric table lookup */
+
+    /* We should have a direct metric table lookup */
     if (!dp->version.two.ext->e_direct)
-	abort();
+	__pmNotifyErr(LOG_INFO, "pmdadarwin: direct metric lookup disabled\n");
 
     mach_host = mach_host_self();
     host_page_size(mach_host, &mach_page_size);
@@ -1232,13 +1235,15 @@ usage(void)
 int
 main(int argc, char **argv)
 {
+    int			sep = __pmPathSeparator();
     int			errflag = 0;
     char		helppath[MAXPATHLEN];
 
     _isDSO = 0;
     __pmSetProgname(argv[0]);
 
-    sprintf(helppath, "%s/pmdas/darwin/help", pmGetConfig("PCP_VAR_DIR"));
+    sprintf(helppath, "%s%c" "darwin" "%c" "help",
+		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
     pmdaDaemon(&dispatch, PMDA_INTERFACE_3, pmProgname, DARWIN, "darwin.log",
 		helppath);
 
