@@ -10,18 +10,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include <unistd.h>
 #include <stdio.h>
-#include <syslog.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <errno.h>
 #include <ctype.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -159,7 +151,7 @@ init_proc_list(void)
     proc_list[0] = (process_t*)malloc(INIT_PROC_MAX * sizeof(process_t));
     proc_list[1] = (process_t*)malloc(INIT_PROC_MAX * sizeof(process_t));
     if (proc_list[0] == NULL || proc_list[1] == NULL || active_list == NULL)
-        return -errno;
+        return -oserror();
     return 0;
 }
 
@@ -193,7 +185,7 @@ add_active_list(process_t *node, config_vars *vars)
 	maxactive = numactive*2;
 	res = (pid_t *)realloc(active_list, maxactive * sizeof(pid_t));
 	if (res == NULL)
-	    return -errno;
+	    return -osoerror();
 	active_list = res;
     } 
     active_list[numactive++] = node->pid;
@@ -209,7 +201,7 @@ dump_active_list(void)
 
    (void)fprintf(stderr, "--- active list ---\n");
    for(i = 0; i < numactive; i++) {
-       (void)fprintf(stderr, "[%d] = %ld\n", i, (long)active_list[i]);
+       (void)fprintf(stderr, "[%d] = %" FMT_PID "\n", i, active_list[i]);
    }/*for*/
    (void)fprintf(stderr, "--- end active list ---\n");
 }
@@ -224,7 +216,7 @@ dump_proc_list(void)
    (void)fprintf(stderr, "--- proc list ---\n");
    for (i = 0; i < numprocs[current]; i++) {
        node = &proc_list[current][i]; 
-       (void)fprintf(stderr, "[%d] = %ld ", i, (long)node->pid);
+       (void)fprintf(stderr, "[%d] = %" FMT_PID " ", i, node->pid);
        (void)fprintf(stderr, "(syscalls = %ld) ", node->r_syscalls); 
        (void)fputc('\n', stderr);
    }/*for*/
@@ -383,8 +375,8 @@ read_buf_file(int *rcount, char *suffix, FILE **testing_file, config_vars *vars)
 	strcat(fname, suffix);
 	*testing_file = fopen(fname, "r"); 
 	if (*testing_file == NULL) {
-	    (void)fprintf(stderr, "%s: Unable to open test file \"%s\": %s\n",
-	    pmProgname, fname, strerror(errno));
+	    fprintf(stderr, "%s: Unable to open test file \"%s\": %s\n",
+	    pmProgname, fname, osstrerror());
 	    return -1;
 	}
     }
@@ -535,7 +527,7 @@ refresh_proc_list(void)
 
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_APPL0) {
-	gettimeofday(&ts);
+	__pmtimevalNow(&ts);
 	curr_time = utime2double(ts);
         fprintf(stderr, "refresh_proc_list():\n");
         __pmProcessRunTimes(&pre_usr, &pre_sys); 
@@ -562,7 +554,7 @@ refresh_proc_list(void)
 	    continue;
 	(void)sscanf(directp->d_name, proc_fmt, &pid);
 
-	(void)gettimeofday(&p_timestamp);
+	__pmtimevalNow(&p_timestamp);
 
         if (psinfo_getbuf_file(pid, &psinfo) != 0)
             continue;
@@ -574,7 +566,7 @@ refresh_proc_list(void)
 	    res = (process_t *)realloc(proc_list[current], 
                                        maxprocs[current] * sizeof(process_t));
 	    if (res == NULL)
-		return -errno;
+		return -oserror();
 	    proc_list[current] = res;
 	} 
 
@@ -622,7 +614,7 @@ refresh_proc_list(void)
 	    if (pmDebug & DBG_TRACE_APPL2) {
 		static char cmd[80];
 		if (cputime_delta > timestamp_delta) {
-		    sprintf(cmd, "(date ; ps -lp %d )>> %s.refresh", pid, log);
+		    sprintf(cmd, "(date ; ps -lp %" FMT_PID " )>> %s.refresh", pid, log);
 		    fprintf(stderr, "refresh: cmd = %s\n", cmd);
 		    system(cmd);
 		}
@@ -734,7 +726,7 @@ refresh_proc_list(void)
 
     numprocs[current] = np;
 
-    (void)gettimeofday(&ts);
+    __pmtimevalNow(&ts);
     refresh_time[current] = utime2double(ts);
 
     refresh_count++;
@@ -746,7 +738,7 @@ refresh_proc_list(void)
 
     bzero(&sinfo, sizeof(sinfo)); /* for purify */
     if ((sysmp_sts = (int)sysmp(MP_SAGET, MPSA_SINFO, &sinfo, sizeof(struct sysinfo))) < 0) {
-   	__pmNotifyErr(LOG_ERR, "sysmp failed in refresh: %s\n", strerror(errno));
+   	__pmNotifyErr(LOG_ERR, "sysmp failed in refresh: %s\n", osstrerror());
         sysidle[current] = -1;
     }
     else {
@@ -848,7 +840,7 @@ hotproc_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmd
 	return PM_ERR_INDOM;
 
     if ((res = (__pmInResult *)malloc(sizeof(*res))) == NULL)
-	return -errno;
+	return -oserror();
     res->indom = indom;
     res->instlist = NULL;
     res->namelist = NULL;
@@ -862,14 +854,14 @@ hotproc_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmd
     if (inst == PM_IN_NULL) {
 	if ((res->instlist = (int *)malloc(res->numinst * sizeof(res->instlist[0]))) == NULL) {
 	    __pmFreeInResult(res);
-	    return -errno;
+	    return -oserror();
 	}
     }
 
     if (name == NULL) {
 	if ((res->namelist = (char **)malloc(res->numinst * sizeof(res->namelist[0]))) == NULL) {
 	    __pmFreeInResult(res);
-	    return -errno;
+	    return -oserror();
 	}
 	for (j = 0; j < res->numinst; j++)
 	    res->namelist[j] = NULL;
@@ -975,7 +967,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
     if (fetched == NULL) {
 	fetched = (int **)malloc(nctltab * sizeof(fetched[0]));
 	if (fetched == NULL)
-	    return -errno;
+	    return -oserror();
         memset(fetched, 0, nctltab * sizeof(fetched[0]));
     }
 
@@ -986,7 +978,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 	/* (numpmid - 1) because there's room for one valueSet in a pmResult */
 	need = sizeof(pmResult) + (numpmid - 1) * sizeof(pmValueSet *);
 	if ((res = (pmResult *) malloc(need)) == NULL)
-	    return -errno;
+	    return -oserror();
 	maxnpmids = numpmid;
     }
     res->timestamp.tv_sec = 0;
@@ -1006,7 +998,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		        free(fetched[ctl_j]);
 		    fetched[ctl_j] = NULL;
 		}
-		return -errno;
+		return -oserror();
             }
 	    fetched[ctl_i] = f;
         }
@@ -1033,7 +1025,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		    res->numpmid = i;
 		    __pmFreeResultValues(res);
 		}
-		return -errno;
+		return -oserror();
 	    }
 	    vset->pmid = pmidlist[i];
 	    vset->numval = pmidErr;
@@ -1049,7 +1041,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		    res->numpmid = i;
 		    __pmFreeResultValues(res);
 		}
-		return -errno;
+		return -oserror();
 	    }
 	    vset->pmid = pmidlist[i];
 	    vset->numval = 1;
@@ -1170,7 +1162,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 #ifdef PCP_DEBUG
 			    if (pmDebug & DBG_TRACE_APPL0) {
 				fprintf(stderr, "hotproc_fetch: getinfo succeeded: "
-					"pid=%d, j=%d\n", pid, j);
+					"pid=%" FMT_PID ", j=%d\n", pid, j);
 			    }
 #endif
 			    fetched[ctl_i][j] = 1;
@@ -1181,7 +1173,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 #ifdef PCP_DEBUG
 			    if (pmDebug & DBG_TRACE_APPL0) {
 				fprintf(stderr, "hotproc_fetch: "
-					"getinfo failed: pid=%d, j=%d\n", pid, j);
+					"getinfo failed: pid=%" FMT_PID ", j=%d\n", pid, j);
 			    }
 #endif
 			    /* remove from active list */
@@ -1213,7 +1205,7 @@ hotproc_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		    res->numpmid = i;
 		    __pmFreeResultValues(res);
 		}
-		return -errno;
+		return -oserror();
 	    }
 	    vset->pmid = pmidlist[i];
 	    vset->numval = numvals;
@@ -1275,7 +1267,7 @@ hotproc_store(pmResult *result, pmdaExt *pmda)
     pmAtomValue av;
 
     if (!allow_stores) {
-	return -EACCES;
+	return PM_ERR_PERMISSION;
     }
 
     for (i = 0; i < result->numpmid; i++) {
@@ -1297,12 +1289,12 @@ hotproc_store(pmResult *result, pmdaExt *pmda)
 		    }
 		    break;
 		default:
-		    sts = -EACCES;
+		    sts = PM_ERR_PERMISSION;
 		    break;
 	    }
 	}
 	else {
-	    sts = -EACCES;
+	    sts = PM_ERR_PERMISSION;
 	}
 
 	if (sts < 0)
@@ -1380,8 +1372,8 @@ hotproc_init(pmdaInterface *dp)
     hotproc_domain = dp->domain;
 
     if ((procdir = opendir(PROCFS)) == NULL) {
-	dp->status = -errno;
-	__pmNotifyErr(LOG_ERR, "opendir(%s) failed: %s\n", PROCFS, strerror(errno));
+	dp->status = -oserror();
+	__pmNotifyErr(LOG_ERR, "opendir(%s) failed: %s\n", PROCFS, osstrerror());
         return;
     }
 
@@ -1389,8 +1381,8 @@ hotproc_init(pmdaInterface *dp)
 
     
     if ((num_cpus = (int)sysmp(MP_NPROCS)) < 0) {
-   	__pmNotifyErr(LOG_ERR, "sysmp failed to get NPROCS: %s\n", strerror(errno));
-	dp->status = -errno;
+	dp->status = -oserror();
+   	__pmNotifyErr(LOG_ERR, "sysmp failed to get NPROCS: %s\n", osstrerror());
         return;
     }
 
@@ -1545,8 +1537,8 @@ main(int argc, char **argv)
 		break;
 	    __pmAFunblock();
 	}
-	else if (nready < 0 && errno != EINTR) {
-	    __pmNotifyErr(LOG_ERR, "select failed: %s\n", strerror(errno));
+	else if (nready < 0 && neterror() != EINTR) {
+	    __pmNotifyErr(LOG_ERR, "select failed: %s\n", netstrerror());
 	}
     }
 
