@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include "pmapi.h"
@@ -44,7 +40,7 @@ acceptClient(int reqfd)
     fd = accept(reqfd, (struct sockaddr *)&clients[i].addr, &addrlen);
     if (fd == -1) {
 	__pmNotifyErr(LOG_ERR, "acceptClient(%d) accept: %s",
-		reqfd, strerror(errno));
+		reqfd, netstrerror());
 	return NULL;
     }
     if (fd > maxfd)
