@@ -14,16 +14,11 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
 #ifndef CLIENT_H
 #define CLIENT_H
 
-#ident "$Id: client.h,v 1.3 1997/09/09 00:13:09 nathans Exp $"
-
 typedef struct {
     int			fd;		/* socket descriptor  */
     struct sockaddr_in	addr;		/* address of client  */
