@@ -10,21 +10,18 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
+#include <inttypes.h>
 #include "pmapi.h"
 #include "impl.h"
 #include "pmlc.h"
 
 /* for the pmlogger/PMCD we currently have a connection to */
-static int	logger_fd = -1;			/* file desc pmlogger */
-static char	*lasthost = NULL;		/* host that logger_ctx is for */
+static int	logger_fd = -1;		/* file desc pmlogger */
+static char	*lasthost;		/* host that logger_ctx is for */
 static int	src_ctx = -1;		/* context for logged host's PMCD*/
-static char	*srchost = NULL;	/* host that logged_ctx is for */
+static char	*srchost;		/* host that logged_ctx is for */
 
 static time_t	tmp;		/* for pmCtime */
 
@@ -63,7 +60,7 @@ ConnectPMCD(void)
 	    fprintf(stderr, "%s\n", pmErrStr(sts));
 	    return sts;
 	}
-	if ((sts = __pmGetPDU(logger_fd, PDU_BINARY, TIMEOUT_NEVER, &pb)) <= 0) {
+	if ((sts = __pmGetPDU(logger_fd, ANY_SIZE, __pmLoggerTimeout(), &pb)) <= 0) {
 	    if (sts == 0)
 		/* end of file! */
 		sts = PM_ERR_IPC;
@@ -78,7 +75,17 @@ ConnectPMCD(void)
 		fprintf(stderr, "%s\n", pmErrStr(sts));
 	    goto done;
 	} 
-	srchost = strdup(lsp->ls_fqdn);
+	if (strcmp(lsp->ls_fqdn, "localhost") == 0) {
+	    /*
+	     * if pmcd host is "localhost" then use hostname that was
+	     * used to contact pmlogger, as from here (where pmlc is
+	     * running) "localhost" is likely to connect us to the wrong
+	     * pmcd or no pmcd at all
+	     */
+	    srchost = strdup(lasthost);
+	}
+	else
+	    srchost = strdup(lsp->ls_fqdn);
     }
 
     if ((sts = pmNewContext(PM_CONTEXT_HOST, srchost)) < 0) {
@@ -279,7 +286,7 @@ Query(void)
 	return;
 
     if ((i = __pmControlLog(logger_fd, logreq, PM_LOG_ENQUIRE, 0, 0, &res)) < 0) {
-	fprintf(stderr, "Error: ");
+	fprintf(stderr, "Error receiving response from pmlogger: ");
 	if (still_connected(i))
 	    fprintf(stderr, "%s\n", pmErrStr(i));
 	return;
@@ -362,7 +369,7 @@ void LogCtl(int control, int state, int delta)
 
     i = __pmControlLog(logger_fd, logreq, control, state, delta, &res);
     if (i < 0 && i != PM_ERR_GENERIC) {
-	fprintf(stderr, "Error: ");
+	fprintf(stderr, "Error receiving response from pmlogger: ");
 	if (still_connected(i))
 	    fprintf(stderr, "%s\n", pmErrStr(i));
 	return;
@@ -533,7 +540,7 @@ void Status(int pid, int primary)
     char		*hostname;
     int			state;
     int			vol;
-    long long		size;
+    __int64_t		size;
     char		startbuf[TZBUFSZ];
     char		lastbuf[TZBUFSZ];
     char		timenowbuf[TZBUFSZ];
@@ -554,7 +561,7 @@ void Status(int pid, int primary)
 		fprintf(stderr, "%s\n", pmErrStr(sts));
 	    return;
 	}
-	if ((sts = __pmGetPDU(logger_fd, PDU_BINARY, TIMEOUT_NEVER, &pb)) <= 0) {
+	if ((sts = __pmGetPDU(logger_fd, ANY_SIZE, __pmLoggerTimeout(), &pb)) <= 0) {
 	    if (sts == 0)
 		/* end of file! */
 		sts = PM_ERR_IPC;
@@ -641,7 +648,7 @@ void Status(int pid, int primary)
 	   "last log entry   %s\n"
 	   "current time     %s\n"
 	   "log volume       %d\n"
-	   "log size         %lld\n",
+	   "log size         %" PRIi64 "\n",
 	   startbuf, zonename, lastbuf, timenowbuf, vol, size);
 
 done:
@@ -672,7 +679,7 @@ Sync(void)
 	}
     }
 
-    if ((sts = __pmGetPDU(logger_fd, PDU_BINARY, TIMEOUT_NEVER, &pb)) != PDU_ERROR) {
+    if ((sts = __pmGetPDU(logger_fd, ANY_SIZE, __pmLoggerTimeout(), &pb)) != PDU_ERROR) {
 	if (sts == 0)
 	    /* end of file! */
 	    sts = PM_ERR_IPC;
@@ -681,7 +688,50 @@ Sync(void)
 	    fprintf(stderr, "%s\n", pmErrStr(sts));
 	return;
     }
-    __pmDecodeError(pb, PDU_BINARY, &sts);
+    __pmDecodeError(pb, &sts);
+    if (sts < 0) {
+	fprintf(stderr, "Error decoding response from pmlogger: ");
+	if (still_connected(sts))
+	    fprintf(stderr, "%s\n", pmErrStr(sts));
+	return;
+    }
+
+    __pmUnpinPDUBuf(pb);
+    return;
+}
+
+void
+Qa(void)
+{
+    int			sts;
+    __pmPDU		*pb;
+
+    if (!connected())
+	return;
+
+    if (__pmVersionIPC(logger_fd) >= LOG_PDU_VERSION2) {
+#ifdef PCP_DEBUG
+	if (pmDebug & DBG_TRACE_PDU)
+	    fprintf(stderr, "pmlc: sending version 2 qa request\n");
+#endif
+	if ((sts = __pmSendLogRequest(logger_fd, 100+qa_case)) < 0) {
+	    fprintf(stderr, "Error sending qa request to pmlogger: ");
+	    if (still_connected(sts))
+		fprintf(stderr, "%s\n", pmErrStr(sts));
+	    return;
+	}
+    }
+
+    if ((sts = __pmGetPDU(logger_fd, ANY_SIZE, __pmLoggerTimeout(), &pb)) != PDU_ERROR) {
+	if (sts == 0)
+	    /* end of file! */
+	    sts = PM_ERR_IPC;
+	fprintf(stderr, "Error receiving response from pmlogger: ");
+	if (still_connected(sts))
+	    fprintf(stderr, "%s\n", pmErrStr(sts));
+	return;
+    }
+    __pmDecodeError(pb, &sts);
     if (sts < 0) {
 	fprintf(stderr, "Error decoding response from pmlogger: ");
 	if (still_connected(sts))
@@ -715,7 +765,7 @@ NewVolume(void)
 	}
     }
 
-    if ((sts = __pmGetPDU(logger_fd, PDU_BINARY, TIMEOUT_NEVER, &pb)) != PDU_ERROR) {
+    if ((sts = __pmGetPDU(logger_fd, ANY_SIZE, __pmLoggerTimeout(), &pb)) != PDU_ERROR) {
 	if (sts == 0)
 	    /* end of file! */
 	    sts = PM_ERR_IPC;
@@ -724,7 +774,7 @@ NewVolume(void)
 	    fprintf(stderr, "%s\n", pmErrStr(sts));
 	return;
     }
-    __pmDecodeError(pb, PDU_BINARY, &sts);
+    __pmDecodeError(pb, &sts);
     if (sts < 0) {
 	fprintf(stderr, "Error decoding response from pmlogger: ");
 	if (still_connected(sts))
@@ -752,11 +802,16 @@ connected(void)
 int
 still_connected(int sts)
 {
-    if (sts == PM_ERR_IPC || sts == PM_ERR_TIMEOUT || sts == -EPIPE) {
+    if (sts == PM_ERR_IPC || sts == -EPIPE) {
 	fprintf(stderr, "Lost connection to the pmlogger instance\n");
 	DisconnectLogger();
 	return 0;
     }
+    if (sts == PM_ERR_TIMEOUT) {
+	fprintf(stderr, "Timeout, closed connection to the pmlogger instance\n");
+	DisconnectLogger();
+	return 0;
+    }
 
     return 1;
 }
