@@ -12,20 +12,11 @@
 # or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 # for more details.
 # 
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-#
-# Example daily administrative script for PCP archive logs
+# Daily administrative script for PCP archive logs
 #
 
-# Get standard environment
 . $PCP_DIR/etc/pcp.env
-
-# Get the portable PCP rc script functions
-if [ -f $PCP_SHARE_DIR/lib/rc-proc.sh ] ; then
-    . $PCP_SHARE_DIR/lib/rc-proc.sh
-fi
+. $PCP_SHARE_DIR/lib/rc-proc.sh
 
 # error messages should go to stderr, not the GUI notifiers
 #
@@ -39,9 +30,12 @@ echo >$tmp.lock
 trap "rm -f \`[ -f $tmp.lock ] && cat $tmp.lock\` $tmp.*; exit \$status" 0 1 2 3 15
 prog=`basename $0`
 
-# 24 hours and 10 minutes ago
-#
-touch -t `pmdate -24H -10M %Y%m%d%H%M` $tmp.merge
+if is_chkconfig_on pcp
+then
+    PMLOGGER_CTL=on
+else
+    PMLOGGER_CTL=off
+fi
 
 # control file for pmlogger administration ... edit the entries in this
 # file to reflect your local configuration (see also -c option below)
@@ -66,9 +60,20 @@ ROLLNOTICES=20480
 # mail addresses to send daily NOTICES summary to
 # 
 MAILME=""
-MAIL=Mail
 MAILFILE=$PCP_LOG_DIR/NOTICES.daily
 
+# search for your mail agent of choice ...
+#
+MAIL=''
+for try in Mail mail email
+do
+    if which $try >/dev/null 2>&1
+    then
+	MAIL=$try
+	break
+    fi
+done
+
 # determine real name for localhost
 LOCALHOST=`hostname | sed -e 's/\..*//'`
 [ -z "$LOCALHOST" ] && LOCALHOST=localhost
@@ -87,6 +92,8 @@ then
     #  force it to a known IRIX location
     PWDCMND=/bin/pwd
 fi
+eval $PWDCMND -P >/dev/null 2>&1
+[ $? -eq 0 ] && PWDCMND="$PWDCMND -P"
 
 _usage()
 {
@@ -95,13 +102,15 @@ Usage: $prog [options]
 
 Options:
   -c control    pmlogger control file
-  -s size       rotate NOTICES file after reaching size bytes
   -k discard    remove archives after "discard" days
   -m addresses  send daily NOTICES entries to email addresses
   -N            show-me mode, no operations performed
   -o            (old style) merge logs only from yesterday
                 [default is to merge all possible logs before today]
-  -V            verbose output
+  -s size       rotate NOTICES file after reaching size bytes
+  -t want       implies -VV and keep trace of verbose output for "want"
+                days
+  -V            verbose output (-VV for very verbose)
   -x compress   compress archive data files after "compress" days
   -X program    use program for archive data file compression
   -Y regex      egrep filter for files to compress ["$COMPRESSREGEX"]
@@ -117,8 +126,9 @@ VERBOSE=false
 VERY_VERBOSE=false
 MYARGS=""
 OFLAG=false
+TRACE=0
 
-while getopts c:k:m:Nos:Vx:X:Y:? c
+while getopts c:k:m:Nos:t:Vx:X:Y:? c
 do
     case $c
     in
@@ -149,6 +159,15 @@ do
 		    exit
 		fi
 		;;
+	t)	TRACE="$OPTARG"
+		# from here on, all stdout and stderr output goes to
+		# $PCP_LOG_DIR/pmlogger/daily.<date>.trace
+		#
+		exec 1>$PCP_LOG_DIR/pmlogger/daily.`date "+%Y%m%d.%H.%M"`.trace 2>&1
+		VERBOSE=true
+		VERY_VERBOSE=true
+		MYARGS="$MYARGS -V -V"
+		;;
 	V)	if $VERBOSE
 		then
 		    VERY_VERBOSE=true
@@ -266,9 +285,18 @@ $1 == "DATE" && $3 == my && $4 == dy && $8 == yy { yday = 1; print; next }
 	    print
 	}' >$tmp.pcp
 
-    [ -s $tmp.pcp ] && \
-	$MAIL -s "PCP NOTICES summary for $LOCALHOST" $MAILME <$tmp.pcp
-    [ -s $tmp.pcp -a -w `dirname $NOTICES` ] && mv $tmp.pcp $MAILFILE
+    if [ -s $tmp.pcp ]
+    then
+	if [ ! -z "$MAIL" ]
+	then
+	    $MAIL -s "PCP NOTICES summary for $LOCALHOST" $MAILME <$tmp.pcp
+	else
+	    echo "$prog: Warning: cannot find a mail agent to send mail ..."
+	    echo "PCP NOTICES summary for $LOCALHOST"
+	    cat $tmp.pcp
+	fi
+        [ -w `dirname $NOTICES` ] && mv $tmp.pcp $MAILFILE
+    fi
 fi
 
 
@@ -455,61 +483,14 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	    continue
 	fi
 
-        case "$PCP_PLATFORM" in
-        irix)
-	    if /sbin/chkconfig pmlogger
-	    then
-		:
-	    else
-		_warning "primary logging disabled via chkconfig for $host"
-		_unlock
-		continue
-	    fi
+	if [ "$PMLOGGER_CTL" = "off" ]
+	then
+	    $VERBOSE && _warning "primary logging disabled for $host"
+	    _unlock
+	    continue
+	fi
 
-	    test -l /var/tmp/pmlogger/primary
-	    ;;
-
-        linux)
-	    #
-	    # On linux, pmcd and (the primary) pmlogger are both controlled
-	    # by the "pcp" chkconfig flag.
-	    #
-	    PMLOGGER_CTL=off
-	    if is_chkconfig_on pcp
-	    then
-		PMLOGGER_CTL=on
-	    fi
-
-	    if [ "$PMLOGGER_CTL" = "off" ]
-	    then
-		_error "primary logging disabled for $host"
-		_unlock
-		continue
-	    fi
-
-	    test -L $PCP_TMP_DIR/pmlogger/primary
-
-	    ;;
-
-        solaris)
-	    PMLOGGER_CTL=off
-	    if is_chkconfig_on pcp
-	    then
-		PMLOGGER_CTL=on
-	    fi
-
-	    if [ "$PMLOGGER_CTL" = "off" ]
-	    then
-		_error "primary logging disabled for $host"
-		_unlock
-		continue
-	    fi
-
-	    test -h $PCP_TMP_DIR/pmlogger/primary
-
-	    ;;
-       
-	esac
+	test -e $PCP_TMP_DIR/pmlogger/primary
 
         if [ $? -eq 0 ]
 	then
@@ -527,14 +508,7 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	fqdn=`pmhostname $host`
 	for log in $PCP_TMP_DIR/pmlogger/[0-9]*
 	do
-	    case "$log"
-	    in
-		*[0-9]*)
-		    # no pmlogger running according to $PCP_TMP_DIR/pmlogger
-		    #
-		    break
-		    ;;
-	    esac
+	    [ "$log" = "$PCP_TMP_DIR/pmlogger/[0-9]*" ] && continue
 	    $VERY_VERBOSE && $PCP_ECHO_PROG $PCP_ECHO_N "... try $log: ""$PCP_ECHO_C"
 	    match=`sed -e '3s/\/[0-9][0-9][0-9][0-9][0-9.]*$//' $log \
 		   | $PCP_AWK_PROG '
@@ -567,7 +541,10 @@ END							{ print m }'`
 
     if [ -z "$pid" ]
     then
-	_error "no pmlogger instance running for host \"$host\""
+	if [ "$PMLOGGER_CTL" = "on" ]
+	then
+	    _error "no pmlogger instance running for host \"$host\""
+	fi
     else
 	if [ "`echo $pid | wc -w`" -gt 1 ]
 	then
@@ -665,13 +642,25 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp.list
 		touch $tmp.skip
 		break
 	    else
+		if $VERY_VERBOSE
+		then
+		    for arch in $inlist
+		    do
+			echo "Input archive $arch ..."
+			pmdumplog -L $arch
+		    done
+		fi
 		if $SHOWME
 		then
 		    echo "+ pmlogger_merge$MYARGS -f $inlist $outfile"
 		else
 		    if pmlogger_merge$MYARGS -f $inlist $outfile
 		    then
-			:
+			if $VERY_VERBOSE
+			then
+			    echo "Merged output archive $outfile ..."
+			    pmdumplog -L $outfile
+			fi
 		    else
 			_error "problems executing pmlogger_merge for host \"$host\""
 		    fi
@@ -711,12 +700,11 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp.list
 	fi
     fi
 
-    # finally, compress old archive data files
+    # and compress old archive data files
     # (after cull - don't compress unnecessarily)
     #
     if [ ! -z "$COMPRESSAFTER" ]
     then
-
 	find . -type f -mtime +$COMPRESSAFTER \
 	| _filter_filename \
 	| egrep -v "$COMPRESSREGEX" \
@@ -737,6 +725,29 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp.list
 	fi
     fi
 
+    # and cull old trace files (from -t option)
+    #
+    if [ "$TRACE" -gt 0 ]
+    then
+	find $PCP_LOG_DIR/pmlogger -type f -mtime +$TRACE \
+	| sed -n -e '/pmlogger\/daily\..*\.trace/p' \
+	| sort >$tmp.list
+	if [ -s $tmp.list ]
+	then
+	    if $VERBOSE
+	    then
+		echo "Trace files older than $TRACE days being removed ..."
+		fmt <$tmp.list | sed -e 's/^/    /'
+	    fi
+	    if $SHOWME
+	    then
+		cat $tmp.list | xargs echo + rm -f
+	    else
+		cat $tmp.list | xargs rm -f
+	    fi
+	fi
+    fi
+
     _unlock
 
 done
