@@ -12,10 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include <math.h>
@@ -104,11 +100,6 @@ metricname(pmID pmid)
 #define MARK_FOR_WRITE		1
 #define WRITTEN			2
 
-#define BREAK_EOF		1
-#define BREAK_LOGEND		2
-#define BREAK_WINEND		3
-
-
 /*
  *  reclist_t is in logger.h
  *	(list of pdu's to write out at start of time window)
@@ -266,7 +257,7 @@ _report(FILE *fp)
     fprintf(stderr, "%s: Error occurred at byte offset %ld into a file of",
 	    pmProgname, (long int)here);
     if (fstat(fileno(fp), &sbuf) < 0)
-	fprintf(stderr, ": stat: %s\n", strerror(errno));
+	fprintf(stderr, ": stat: %s\n", osstrerror());
     else
 	fprintf(stderr, " %ld bytes.\n", (long int)sbuf.st_size);
     fprintf(stderr, "The last record, and the remainder of this file will not be extracted.\n");
@@ -282,25 +273,26 @@ newvolume(char *base, __pmTimeval *tvp)
 {
     FILE		*newfp;
     int			nextvol = logctl.l_curvol + 1;
-    struct timeval	stamp;
 
     if ((newfp = __pmLogNewFile(base, nextvol)) != NULL) {
+	struct timeval	stamp;
 	fclose(logctl.l_mfp);
 	logctl.l_mfp = newfp;
 	logctl.l_label.ill_vol = logctl.l_curvol = nextvol;
 	__pmLogWriteLabel(logctl.l_mfp, &logctl.l_label);
 	fflush(logctl.l_mfp);
-	stamp.tv_sec = tvp->tv_sec;
-	stamp.tv_usec = tvp->tv_usec;
+	stamp.tv_sec = ntohl(tvp->tv_sec);
+	stamp.tv_usec = ntohl(tvp->tv_usec);
 	fprintf(stderr, "%s: New log volume %d, at ", pmProgname, nextvol);
 	__pmPrintStamp(stderr, &stamp);
 	fputc('\n', stderr);
     }
     else {
 	fprintf(stderr, "%s: Error: volume %d: %s\n",
-		pmProgname, nextvol, pmErrStr(-errno));
+		pmProgname, nextvol, pmErrStr(-oserror()));
 	abandon();
     }
+    flushsize = 100000;
 }
 
 
@@ -330,7 +322,7 @@ newlabel(void)
 
     /* copy magic number, pid, host and timezone */
     lp->ill_magic = iap->label.ll_magic;
-    lp->ill_pid = getpid();
+    lp->ill_pid = (int)getpid();
     strncpy(lp->ill_hostname, iap->label.ll_hostname, PM_LOG_MAXHOSTLEN);
     if (farg) {
 	/*
@@ -768,7 +760,7 @@ write_metareclist(pmResult *result, int *needti)
 	if (curr_desc == NULL) {
 	    /* descriptor has not been found - this is bad
 	     */
-	    fprintf(stderr, "%s: Error: meta data (TYPE_DESC) for pmid %d has not been found.\n", pmProgname, pmid);
+	    fprintf(stderr, "%s: Error: meta data (TYPE_DESC) for pmid %s has not been found.\n", pmProgname, pmIDStr(pmid));
 	    abandon();
 	}
 	else {
@@ -785,8 +777,8 @@ write_metareclist(pmResult *result, int *needti)
 		/* descriptor is in list, has not been written, but no pdu
 		 *  - this is bad
 		 */
-		fprintf(stderr, "%s: Error: missing pdu for pmid %d\n",
-			pmProgname, pmid);
+		fprintf(stderr, "%s: Error: missing pdu for pmid %s\n",
+			pmProgname, pmIDStr(pmid));
 	        abandon();
 	    }
 	    else {
@@ -870,7 +862,7 @@ _createmark(void)
     markp = (mark_t *)malloc(sizeof(mark_t));
     if (markp == NULL) {
 	fprintf(stderr, "%s: Error: mark_t malloc: %s\n",
-		pmProgname, strerror(errno));
+		pmProgname, osstrerror());
 	abandon();
     }
 #ifdef PCP_DEBUG
@@ -1292,7 +1284,7 @@ parseconfig(void)
 
     if ((yyin = fopen(configfile, "r")) == NULL) {
 	fprintf(stderr, "%s: Cannot open config file \"%s\": %s\n",
-		pmProgname, configfile, strerror(errno));
+		pmProgname, configfile, osstrerror());
 	exit(1);
     }
 
@@ -1430,6 +1422,7 @@ writerlist(rlist_t **rlready, double mintime)
     rlist_t	*elm;		/* element of rlready to be written out */
     __pmPDU	*pb;		/* pdu buffer */
     __pmTimeval	this;		/* timeval of this record */
+    unsigned long	peek_offset;
 
     needti = 0;
     titime = 0.0;
@@ -1467,11 +1460,6 @@ writerlist(rlist_t **rlready, double mintime)
 	    pre_startwin = 0;
 
 
-	/* write out the descriptor and instance domain pdu's first
-	 */
-	write_metareclist(elm->res, &needti);
-
-
 	/* convert log record to a pdu
 	 */
 	if (outarchvers == 1)
@@ -1489,8 +1477,29 @@ writerlist(rlist_t **rlready, double mintime)
 	 */
 	__pmPinPDUBuf(pb);
 
+        /* switch volumes if required */
+        if (varg > 0) {
+            if (written > 0 && (written % varg) == 0) {
+                newvolume(outarchname, (__pmTimeval *)&pb[3]);
+	    }
+        }
+	/*
+	 * Even without a -v option, we may need to switch volumes
+	 * if the data file exceeds 2^31-1 bytes
+	 */
+	peek_offset = ftell(logctl.l_mfp);
+	peek_offset += ((__pmPDUHdr *)pb)->len - sizeof(__pmPDUHdr) + 2*sizeof(int);
+	if (peek_offset > 0x7fffffff) {
+	    newvolume(outarchname, (__pmTimeval *)&pb[3]);
+	}
+
+	/* write out the descriptor and instance domain pdu's first
+	 */
+	write_metareclist(elm->res, &needti);
+
 
 	/* write out log record */
+	old_log_offset = ftell(logctl.l_mfp);
 	if ((sts = __pmLogPutResult(&logctl, pb)) < 0) {
 	    fprintf(stderr, "%s: Error: __pmLogPutResult: log data: %s\n",
 		    pmProgname, pmErrStr(sts));
@@ -1538,12 +1547,6 @@ writerlist(rlist_t **rlready, double mintime)
             flushsize = ftell(logctl.l_mfp) + 100000;
         }
 
-        /* switch volumes if required */
-        if (varg > 0) {
-            if (written % varg == 0)
-                newvolume(outarchname, (__pmTimeval *)&pb[3]);
-        }
-
 	/* LOG: free PDU buffer */
 	__pmUnpinPDUBuf(pb);
 	pb = NULL;
@@ -1604,7 +1607,6 @@ main(int argc, char **argv)
     int		sts;
     int		stslog;			/* sts from nextlog() */
     int		stsmeta;		/* sts from nextmeta() */
-    int		breakflag;		/* reason for breaking out of while */
 
     char	*msg;
 
@@ -1644,7 +1646,7 @@ main(int argc, char **argv)
     inarch = (inarch_t *) malloc(inarchnum * sizeof(inarch_t));
     if (inarch == NULL) {
 	fprintf(stderr, "%s: Error: mallco inarch: %s\n",
-		pmProgname, strerror(errno));
+		pmProgname, osstrerror());
 	exit(1);
     }
 #ifdef PCP_DEBUG
@@ -1816,7 +1818,6 @@ main(int argc, char **argv)
     current.tv_usec = 0;
     first_datarec = 1;
     pre_startwin = 1;
-    breakflag = 0;
 
     /* get all meta data first
      * nextmeta() should return 0 (will return -1 when all meta is eof)
@@ -1835,17 +1836,14 @@ main(int argc, char **argv)
 	ilog = -1;
 	curlog.tv_sec = 0;
 	curlog.tv_usec = 0;
-	old_log_offset = ftell(logctl.l_mfp);
 	old_meta_offset = ftell(logctl.l_mdfp);
 
 	/* nextlog() resets ilog, and curlog (to the smallest timestamp)
 	 */
 	stslog = nextlog();
 
-	if (stslog < 0) {
-	    breakflag = BREAK_EOF;
+	if (stslog < 0)
 	    break;
-	}
 
 	/* find the _Nresult (or mark pdu) with the earliest timestamp;
 	 * set ilog
@@ -1887,16 +1885,12 @@ main(int argc, char **argv)
 	/* note - mark (after last archive) will be created, but this
 	 * break, will prevent it from being written out
 	 */
-	if (now > logend_time) {
-	    breakflag = BREAK_LOGEND;
+	if (now > logend_time)
 	    break;
-	}
 
 	sts = checkwinend(now);
-	if (sts < 0) {
-	    breakflag = BREAK_WINEND;
+	if (sts < 0)
 	    break;
-	}
 	if (sts > 0)
 	    continue;
 
@@ -1976,6 +1970,7 @@ cleanup:
 	    mintime, tmptime, logend_time, winend_time, current.tv_sec, current.tv_usec);
 #endif
 
+	fseek(logctl.l_mfp, old_log_offset, SEEK_SET);
 	__pmLogPutIndex(&logctl, &current);
 
 
