@@ -16,8 +16,6 @@
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
-#include "pmapi.h"
-#include "impl.h"
 #include "logger.h"
 
 int
@@ -57,16 +55,30 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 	if (pmDebug & DBG_TRACE_PROFILE)
 	    fprintf(stderr, "myFetch: calling __pmSendProfile, context: %d\n", ctx);
 #endif
-	if ((n = __pmSendProfile(ctxp->c_pmcd->pc_fd, PDU_BINARY, ctx, ctxp->c_instprof)) >= 0)
+	if ((n = __pmSendProfile(ctxp->c_pmcd->pc_fd, FROM_ANON, ctx, ctxp->c_instprof)) >= 0)
 	    ctxp->c_sent = 1;
     }
 
     if (n >= 0) {
-	n = __pmSendFetch(ctxp->c_pmcd->pc_fd, PDU_BINARY, ctx, &ctxp->c_origin, numpmid, pmidlist);
+	int		newcnt;
+	pmID		*newlist;
+	int		have_dm;
+
+	/* for derived metrics, may need to rewrite the pmidlist */
+	have_dm = newcnt = __dmprefetch(ctxp, numpmid, pmidlist, &newlist);
+	if (newcnt > numpmid) {
+	    /* replace args passed into myFetch */
+	    numpmid = newcnt;
+	    pmidlist = newlist;
+	}
+	else
+	    newlist = NULL;
+
+	n = __pmSendFetch(ctxp->c_pmcd->pc_fd, FROM_ANON, ctx, &ctxp->c_origin, numpmid, pmidlist);
 	if (n >= 0){
 	    int		changed = 0;
 	    do {
-		n = __pmGetPDU(ctxp->c_pmcd->pc_fd, PDU_BINARY, TIMEOUT_DEFAULT, &pb);
+		n = __pmGetPDU(ctxp->c_pmcd->pc_fd, ANY_SIZE, TIMEOUT_DEFAULT, &pb);
 		/*
 		 * expect PDU_RESULT or
 		 *        PDU_ERROR(changed > 0)+PDU_RESULT or
@@ -75,11 +87,39 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 		 *        < 0 (local error or IPC problem)
 		 *        other (bogus PDU)
 		 */
-		if (n == PDU_RESULT)
-		    /* success */
-		    *pdup = pb;
+		if (n == PDU_RESULT) {
+		    /*
+		     * Success with a pmResult in a pdubuf.
+		     *
+		     * Need to process derived metrics, if any.
+		     * This is ugly, we need to decode the pdubuf, rebuild
+		     * the pmResult and encode back into a pdubuf ... the
+		     * fastpath of not doing all of this needs to be
+		     * preserved in the common case where derived metrics
+		     * are not being logged.
+		     */
+		    if (have_dm) {
+			pmResult	*result;
+			__pmPDU		*npb;
+			int		sts;
+			if ((sts = __pmDecodeResult(pb, &result)) < 0) {
+			    n = sts;
+			}
+			else {
+			    __dmpostfetch(ctxp, &result);
+			    __pmPinPDUBuf(pb);
+			    if ((sts = __pmEncodeResult(ctxp->c_pmcd->pc_fd, result, &npb)) < 0)
+				n = sts;
+			    else
+				*pdup = npb;
+			    __pmUnpinPDUBuf(pb);
+			}
+		    }
+		    else
+			*pdup = pb;
+		}
 		else if (n == PDU_ERROR) {
-		    __pmDecodeError(pb, PDU_BINARY, &n);
+		    __pmDecodeError(pb, &n);
 		    if (n > 0) {
 			/* PMCD state change protocol */
 			changed = n;
@@ -123,6 +163,8 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 		}
 	    }
 	}
+	if (newlist != NULL)
+	    free(newlist);
     }
 
     if (n < 0 && ctxp->c_pmcd->pc_fd != -1) {
