@@ -20,6 +20,7 @@
 #define _LOGGER_H
 
 #include "pmapi.h"
+#include "impl.h"
 
 /*
  * a task is a bundle of fetches to be done together
@@ -44,8 +45,9 @@ extern __pmLogCtl	logctl;		/* global log control */
 #define GLOBAL  0
 #define INSPEC  1
 
-/* generic error message buffer */
-extern char     emess[];
+/* generic error messages */
+extern char	*chk_emess[];
+extern void die(char *, int);
 
 /*
  * hash control for per-metric (really per-metric per-log specification)
@@ -90,6 +92,7 @@ typedef struct {
 #define VOL_SW_COUNTER 2
 #define VOL_SW_BYTES   3
 #define VOL_SW_TIME    4
+#define VOL_SW_MAX     5
 
 /* flush requested via SIGUSR1 */
 extern int	wantflush;
@@ -100,8 +103,14 @@ extern int	unbuffered;
 /* initial time of day from remote PMCD */
 extern struct timeval	epoch;
 
+/* offset to start of last written pmResult */
+extern int	last_log_offset;
+
 /* yylex() gets input from here ... */
 extern FILE		*fconfig;
+extern FILE		*yyin;
+extern char		*configfile;
+extern int		lineno;
 
 extern int myFetch(int, pmID *, __pmPDU **);
 extern void yyerror(char *);
@@ -133,9 +142,33 @@ extern char pmlc_host[];
 #define LOG_DELTA_DEFAULT	-2
 
 /* command line parameters */
-extern char    	*archBase;		/* base name for log files */
-extern char	*pmcd_host;		/* collecting from PMCD on this host */
-extern int	primary;		/* Non-zero for primary logger */
+extern char	    	*archBase;		/* base name for log files */
+extern char		*pmcd_host;		/* collecting from PMCD on this host */
+extern int		primary;		/* Non-zero for primary logger */
+extern int		rflag;
+extern struct timeval	delta;			/* default logging interval */
+extern int		ctlport;		/* pmlogger control port number */
+
+/* pmlc support */
+extern void init_ports(void);
+extern int control_req(void);
+extern int client_req(void);
+extern __pmHashCtl	hist_hash;
+extern unsigned int	clientops;	/* access control (deny ops) */
+extern struct timeval	last_stamp;
+extern int		clientfd;
+extern int		ctlfd;
+extern int		exit_samples;
+extern int		vol_switch_samples;
+extern __int64_t	vol_switch_bytes;
+extern int		vol_samples_counter;
+extern int		archive_version; 
+extern int		parse_done;
+extern __int64_t	exit_bytes;
+extern __int64_t	vol_bytes;
+
+/* event record handling */
+extern int do_events(pmValueSet *);
 
 /* QA testing and error injection support ... see do_request() */
 extern int	qa_case;
