@@ -10,41 +10,35 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include <ctype.h>
 #include <limits.h>
 #include <sys/stat.h>
-#include "pmapi.h"
-#include "impl.h"
 #include "logger.h"
 
-char		*configfile = NULL;
+char		*configfile;
 __pmLogCtl	logctl;
 int		exit_samples = -1;       /* number of samples 'til exit */
-long		exit_bytes = -1;         /* number of bytes 'til exit */
-long		vol_bytes = 0;		/* total in earlier volumes */
+__int64_t	exit_bytes = -1;         /* number of bytes 'til exit */
+__int64_t	vol_bytes;		 /* total in earlier volumes */
 struct timeval  exit_time;               /* time interval 'til exit */
 int		vol_switch_samples = -1; /* number of samples 'til vol switch */
-long		vol_switch_bytes = -1;   /* number of bytes 'til vol switch */
+__int64_t	vol_switch_bytes = -1;   /* number of bytes 'til vol switch */
 struct timeval	vol_switch_time;         /* time interval 'til vol switch */
-int		vol_samples_counter = 0; /* Counts samples - reset for new vol*/
+int		vol_samples_counter;     /* Counts samples - reset for new vol*/
 int		vol_switch_afid = -1;    /* afid of event for vol switch */
-int		parse_done = 0;
-int		primary = 0;		/* Non-zero for primary pmlogger */
+int		parse_done;
+int		primary;		/* Non-zero for primary pmlogger */
 char	    	*archBase;		/* base name for log files */
-char		*pmcd_host = NULL;
+char		*pmcd_host;
 struct timeval	epoch;
 int		archive_version = PM_LOG_VERS02; /* Type of archive to create */
-int		linger = 0;		/* linger with no tasks/events */
-int		rflag = 0;		/* report sizes */
+int		linger;			/* linger with no tasks/events */
+int		rflag;			/* report sizes */
 struct timeval	delta = { 60, 0 };	/* default logging interval */
-int		unbuffered = 0;		/* is -u specified? */
-int		qa_case = 0;		/* QA error injection state */
+int		unbuffered;		/* is -u specified? */
+int		qa_case;		/* QA error injection state */
 
 static int 	    pmcdfd;		/* comms to pmcd */
 static int	    ctx;		/* handle correspondong to ctxp below */
@@ -52,11 +46,8 @@ static __pmContext  *ctxp;		/* pmlogger has just this one context */
 static fd_set	    fds;		/* file descriptors mask for select */
 static int	    numfds;		/* number of file descriptors in mask */
 
-extern struct timeval	last_stamp;
-extern int		ctlfd, clientfd;
-
 static int	rsc_fd = -1;	/* recording session control see -x */
-static int	rsc_replay = 0;
+static int	rsc_replay;
 static time_t	rsc_start;
 static char	*rsc_prog = "<unknown>";
 static char	*folio_name = "<unknown>";
@@ -72,11 +63,11 @@ do_flush(void)
 
     sts = 0;
     if (fflush(logctl.l_mdfp) != 0)
-	sts = errno;
+	sts = oserror();
     if (fflush(logctl.l_mfp) != 0 && sts == 0)
-	sts = errno;
+	sts = oserror();
     if (fflush(logctl.l_tifp) != 0 && sts == 0)
-	sts = errno;
+	sts = oserror();
 
     return sts;
 }
@@ -91,11 +82,16 @@ run_done(int sts, char *msg)
     	fprintf(stderr, "pmlogger: End of run time, exiting\n");
 #endif
 
-    /* write the last time stamp */
+    /*
+     * write the last last temportal index entry with the time stamp
+     * of the last pmResult and the seek pointer set to the offset
+     * _before_ the last log record
+     */
     if (last_stamp.tv_sec != 0) {
 	__pmTimeval	tmp;
 	tmp.tv_sec = (__int32_t)last_stamp.tv_sec;
 	tmp.tv_usec = (__int32_t)last_stamp.tv_usec;
+	fseek(logctl.l_mfp, last_log_offset, SEEK_SET);
 	__pmLogPutIndex(&logctl, &tmp);
     }
 
@@ -152,10 +148,10 @@ tolower_str(char *str)
  *
  */
 static int
-ParseSize(char *size_arg, int *sample_counter, long *byte_size, 
+ParseSize(char *size_arg, int *sample_counter, __int64_t *byte_size, 
           struct timeval *time_delta)
 {
-  double x = 0; /* the size number */
+  long x = 0; /* the size number */
   char *ptr = NULL;
 
   *sample_counter = -1;
@@ -163,7 +159,7 @@ ParseSize(char *size_arg, int *sample_counter, long *byte_size,
   time_delta->tv_sec = -1;
   time_delta->tv_usec = -1;
   
-  x = strtod(size_arg, &ptr);
+  x = strtol(size_arg, &ptr, 10);
 
   /* must be positive */
   if (x <= 0) {
@@ -192,25 +188,37 @@ ParseSize(char *size_arg, int *sample_counter, long *byte_size,
     /* if bytes */
     if (strcmp(ptr, "b") == 0 ||
         strcmp(ptr, "byte") == 0) {
-      *byte_size = (long)x;
+      *byte_size = x;
       return 1;
     }  
 
     /* if kilobytes */
     if (strcmp(ptr, "k") == 0 ||
         strcmp(ptr, "kb") == 0 ||
+        strcmp(ptr, "kbyte") == 0 ||
         strcmp(ptr, "kilobyte") == 0) {
-      *byte_size = (long)(x*1024);
+      *byte_size = x*1024;
       return 1;
     }
 
     /* if megabytes */
     if (strcmp(ptr, "m") == 0 ||
         strcmp(ptr, "mb") == 0 ||
+        strcmp(ptr, "mbyte") == 0 ||
         strcmp(ptr, "megabyte") == 0) {
-      *byte_size = (long)(x*1024*1024);
+      *byte_size = x*1024*1024;
       return 1;
     }
+
+    /* if gigabytes */
+    if (strcmp(ptr, "g") == 0 ||
+        strcmp(ptr, "gb") == 0 ||
+        strcmp(ptr, "gbyte") == 0 ||
+        strcmp(ptr, "gigabyte") == 0) {
+      *byte_size = ((__int64_t)x)*1024*1024*1024;
+      return 1;
+    }
+
   }
   
   /* Doesn't fit pattern above, try a time interval */
@@ -378,7 +386,7 @@ do_dialog(char cmd)
     if (cmd != 'Q') {
 	nchar = add_msg(&p, nchar, "\n\nAt any time this pmlogger process may be terminated with the");
 	nchar = add_msg(&p, nchar, " following command:\n");
-	sprintf(lbuf, "  $ pmsignal -s TERM %d\n", (int)getpid());
+	sprintf(lbuf, "  $ pmsignal -s TERM %" FMT_PID "\n", getpid());
 	nchar = add_msg(&p, nchar, lbuf);
     }
 
@@ -401,7 +409,7 @@ do_dialog(char cmd)
 	    msgf = fdopen(fd, "w");
 	if (msgf == NULL) {
 	    fprintf(stderr, "\nError: failed create temporary message file for recording session dialog\n");
-	    fprintf(stderr, "Reason? %s\n", strerror(errno));
+	    fprintf(stderr, "Reason? %s\n", osstrerror());
 	    if (fd != -1)
 		close(fd);
 	    goto failed;
@@ -446,7 +454,7 @@ failed:
     }
     else {
 	fprintf(stderr, "Error: failed to create recording session dialog message!\n");
-	fprintf(stderr, "Reason? %s\n", strerror(errno));
+	fprintf(stderr, "Reason? %s\n", osstrerror());
 	strcpy(lbuf, "Yes");
     }
 
@@ -480,7 +488,6 @@ main(int argc, char **argv)
     int			i;
     task_t		*tp;
     optcost_t		ocp;
-    extern void		init_ports(void);
     fd_set		readyfds;
     char		*p;
     char		*runtime = NULL;
@@ -684,22 +691,7 @@ Options:\n\
 
     if (rsc_fd == -1) {
 	/* no -x, so register client id with pmcd */
-	int	need = 0;
-	int	a;
-	char	*ip;
-	for (a = 0; a < argc; a++)
-	    need += strlen(argv[a])+1;
-	ip = (char *)malloc(need);
-	if (ip != NULL) {
-	    ip[0] = '\0';
-	    for (a = 0; a < argc; a++) {
-		strcat(ip, argv[a]);
-		if (a < argc-1)
-		    strcat(ip, " ");
-	    }
-	    __pmSetClientId(ip);
-	    free(ip);
-	}
+	__pmSetClientIdArgv(argc, argv);
     }
 
     /*
@@ -707,12 +699,12 @@ Options:\n\
      */
     ctxp = __pmHandleToPtr(ctx);
     pmcdfd = ctxp->c_pmcd->pc_fd;
+    pmcd_host = ctxp->c_pmcd->pc_hosts[0].name;
 
     if (configfile != NULL) {
-	extern FILE * yyin;
 	if ((yyin = fopen(configfile, "r")) == NULL) {
 	    fprintf(stderr, "%s: Cannot open config file \"%s\": %s\n",
-		pmProgname, configfile, strerror(errno));
+		pmProgname, configfile, osstrerror());
 	    exit(1);
 	}
     }
@@ -732,7 +724,6 @@ Options:\n\
 	exit(1);
 
     if ( configfile != NULL ) {
-	extern FILE * yyin;
 	fclose(yyin);
     }
 
@@ -755,7 +746,7 @@ Options:\n\
 		PMLC_GET_AVAIL(tp->t_state) ? "" : "un",
 		PMLC_GET_MAND(tp->t_state) ? "mand" : "adv",
 		PMLC_GET_ON(tp->t_state) ? "on" : "off");
-	    fprintf(stderr, " delta: %ld msec", 
+	    fprintf(stderr, " delta: %ld usec", 
 			(long)1000 * tp->t_delta.tv_sec + tp->t_delta.tv_usec);
 	    fprintf(stderr, " numpmid: %d\n", tp->t_numpmid);
 	    for (i = 0; i < tp->t_numpmid; i++) {
@@ -784,7 +775,7 @@ Options:\n\
 	pmID		pmid;
 	pmResult	*resp;
 
-	gettimeofday(&epoch, NULL);
+	__pmtimevalNow(&epoch);
 	sts = pmUseContext(ctx);
 	if (sts >= 0)
 	    sts = pmLookupName(1, &name, &pmid);
@@ -848,7 +839,9 @@ Options:\n\
     init_ports();
     FD_ZERO(&fds);
     FD_SET(ctlfd, &fds);
+#ifndef IS_MINGW
     FD_SET(pmcdfd, &fds);
+#endif
     if (rsc_fd != -1)
 	FD_SET(rsc_fd, &fds);
     numfds = maxfd() + 1;
@@ -864,8 +857,6 @@ Options:\n\
 
     for ( ; ; ) {
 	int		nready;
-	extern int	control_req(void);
-	extern int	client_req(void);
 
 	memcpy(&readyfds, &fds, sizeof(readyfds));
 	nready = select(numfds, &readyfds, NULL, NULL, NULL);
@@ -904,14 +895,19 @@ Options:\n\
 		    qa_case = 0;
 		}
 	    }
+#ifndef IS_MINGW
 	    if (pmcdfd >= 0 && FD_ISSET(pmcdfd, &readyfds)) {
 		/*
 		 * do not expect this, given synchronous commumication with the
-		 * pmcd ... either pmcd has terminated, or bogus PDU ...
+		 * pmcd ... either pmcd has terminated, or bogus PDU ... or its
+		 * Win32 and we are operating under the different conditions of
+		 * our AF.c implementation there, which has to deal with a lack
+		 * of signal support on Windows - race condition exists between
+		 * this check and the async event timer callback.
 		 */
 		__pmPDU		*pb;
 		__pmPDUHdr	*php;
-		sts = __pmGetPDU(pmcdfd, PDU_BINARY, TIMEOUT_NEVER, &pb);
+		sts = __pmGetPDU(pmcdfd, ANY_SIZE, TIMEOUT_NEVER, &pb);
 		if (sts <= 0) {
 		    if (sts < 0)
 			fprintf(stderr, "Error: __pmGetPDU: %s\n", pmErrStr(sts));
@@ -924,6 +920,7 @@ Options:\n\
 		    disconnect(PM_ERR_IPC);
 		}
 	    }
+#endif
 	    if (rsc_fd >= 0 && FD_ISSET(rsc_fd, &readyfds)) {
 		/*
 		 * some action on the recording session control fd
@@ -1000,8 +997,8 @@ Options:\n\
 
 	    __pmAFunblock();
 	}
-	else if (nready < 0 && errno != EINTR)
-	    perror("select");
+	else if (nready < 0 && neterror() != EINTR)
+	    fprintf(stderr, "Error: select: %s\n", netstrerror());
     }
 
 }
@@ -1015,7 +1012,7 @@ newvolume(int vol_switch_type)
     time_t	now;
     static char *vol_sw_strs[] = {
        "SIGHUP", "pmlc request", "sample counter",
-       "sample byte size", "sample time"
+       "sample byte size", "sample time", "max data volume size"
     };
 
     vol_samples_counter = 0;
@@ -1043,7 +1040,7 @@ newvolume(int vol_switch_type)
 	    /*
 	     * nothing has been logged as yet, force out the label records
 	     */
-	    gettimeofday(&last_stamp, NULL);
+	    __pmtimevalNow(&last_stamp);
 	    logctl.l_label.ill_start.tv_sec = (__int32_t)last_stamp.tv_sec;
 	    logctl.l_label.ill_start.tv_usec = (__int32_t)last_stamp.tv_usec;
 	    logctl.l_label.ill_vol = PM_LOG_VOL_TI;
@@ -1073,7 +1070,7 @@ newvolume(int vol_switch_type)
 	return nextvol;
     }
     else
-	return -errno;
+	return -oserror();
 }
 
 
