@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include "pmapi.h"
@@ -41,23 +37,23 @@ verify_label(FILE *f, const char *file)
     len = ntohl(len);
     if (n != sizeof(len)) {
 	if (feof(f)) {
-	    fprintf(stderr, "Bad prefix sentinal read for %s: file too short\n",
+	    fprintf(stderr, "Bad prefix sentinel read for %s: file too short\n",
 			file);
 	    status = 2;
 	}
 	else if (ferror(f)) {
-	    fprintf(stderr, "Prefix sentinal read error for %s: %s\n",
-			file, strerror(errno));
+	    fprintf(stderr, "Prefix sentinel read error for %s: %s\n",
+			file, osstrerror());
 	    status = 2;
 	}
 	else {
-	    fprintf(stderr, "Prefix sentinal read error for %s: read only %d\n",
+	    fprintf(stderr, "Prefix sentinel read error for %s: read only %d\n",
 			file, n);
 	    status = 2;
 	}
     }
     if (len != xpectlen) {
-	fprintf(stderr, "Bad prefix sentinal value for %s: %d (%d expected)\n",
+	fprintf(stderr, "Bad prefix sentinel value for %s: %d (%d expected)\n",
 			file, len, xpectlen);
 	status = 2;
     }
@@ -68,23 +64,23 @@ verify_label(FILE *f, const char *file)
     len = ntohl(len);
     if (n != sizeof(len)) {
 	if (feof(f)) {
-	    fprintf(stderr, "Bad suffix sentinal read for %s: file too short\n",
+	    fprintf(stderr, "Bad suffix sentinel read for %s: file too short\n",
 			file);
 	    status = 2;
 	}
 	else if (ferror(f)) {
-	    fprintf(stderr, "Suffix sentinal read error for %s: %s\n",
-			file, strerror(errno));
+	    fprintf(stderr, "Suffix sentinel read error for %s: %s\n",
+			file, osstrerror());
 	    status = 2;
 	}
 	else {
-	    fprintf(stderr, "Suffix sentinal read error for %s: read only %d\n",
+	    fprintf(stderr, "Suffix sentinel read error for %s: read only %d\n",
 			file, n);
 	    status = 2;
 	}
     }
     if (len != xpectlen) {
-	fprintf(stderr, "Bad suffix sentinal value for %s: %d (%d expected)\n",
+	fprintf(stderr, "Bad suffix sentinel value for %s: %d (%d expected)\n",
 			file, len, xpectlen);
 	status = 2;
     }
@@ -128,7 +124,7 @@ compare_golden(FILE *f, const char *file, int sts, int warnings)
 	    status = 2;
 	}
 	if (label->ill_pid != golden.ill_pid) {
-	    fprintf(stderr, "Mismatched PID (%u/%u) between %s and %s\n",
+	    fprintf(stderr, "Mismatched PID (%d/%d) between %s and %s\n",
 			    label->ill_pid, golden.ill_pid, file, goldfile);
 	    status = 2;
 	}
@@ -153,7 +149,6 @@ main(int argc, char *argv[])
     int			sts;
     int			lflag = 0;
     int			Lflag = 0;
-    int			sflag = 0;
     int			errflag = 0;
     int			verbose = 0;
     int			version = 0;
@@ -199,8 +194,7 @@ main(int argc, char *argv[])
 	    readonly = 0;
 	    break;
 
-	case 's':	/* rewrite sentinals */
-	    sflag = 1;
+	case 's':	/* rewrite sentinels */
 	    readonly = 0;
 	    break;
 
@@ -239,7 +233,7 @@ main(int argc, char *argv[])
 "  -l           dump the archive label\n"
 "  -L           more verbose form of -l\n"
 "  -p pid       set the logger process ID field for all files in archive\n"
-"  -s           write the label sentinal values for all files in archive\n"
+"  -s           write the label sentinel values for all files in archive\n"
 "  -v           run in verbose mode, reporting on each stage of checking\n"
 "  -V version   write magic and version numbers for all files in archive\n"
 "  -Z timezone  set the timezone for all files in archive\n",
@@ -326,7 +320,7 @@ main(int argc, char *argv[])
 	    snprintf(buffer, sizeof(buffer), "%s.%d", logctl.l_name, c);
 	    if ((logctl.l_mfp = fopen(buffer, "r+")) == NULL) {
 		fprintf(stderr, "Failed data volume %d open: %s\n",
-				c, strerror(errno));
+				c, osstrerror());
 		status = 3;
 	    }
 	    else if ((sts = __pmLogWriteLabel(logctl.l_mfp, &golden)) < 0) {
@@ -350,7 +344,7 @@ main(int argc, char *argv[])
 	    snprintf(buffer, sizeof(buffer), "%s.index", logctl.l_name);
 	    if ((logctl.l_tifp = fopen(buffer, "r+")) == NULL) {
 		fprintf(stderr, "Failed temporal index open: %s\n",
-				strerror(errno));
+				osstrerror());
 		status = 3;
 	    }
 	    else if ((sts = __pmLogWriteLabel(logctl.l_tifp, &golden)) < 0) {
@@ -367,7 +361,7 @@ main(int argc, char *argv[])
 	snprintf(buffer, sizeof(buffer), "%s.meta", logctl.l_name);
 	if ((logctl.l_mdfp = fopen(buffer, "r+")) == NULL) {
 	    fprintf(stderr, "Failed metadata volume open: %s\n",
-			    strerror(errno));
+			    osstrerror());
 	    status = 3;
 	}
 	else if ((sts = __pmLogWriteLabel(logctl.l_mdfp, &golden)) < 0) {
@@ -384,11 +378,12 @@ main(int argc, char *argv[])
 	char	       *ddmm;
 	char	       *yr;
 	struct timeval	tv;
+	time_t t = golden.ill_start.tv_sec;
 
 	printf("Log Label (Log Format Version %d)\n", golden.ill_magic & 0xff);
 	printf("Performance metrics from host %s\n", golden.ill_hostname);
 
-	ddmm = pmCtime((const time_t *)&golden.ill_start.tv_sec, buffer);
+	ddmm = pmCtime(&t, buffer);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("  commencing %s ", ddmm);
@@ -401,7 +396,7 @@ main(int argc, char *argv[])
 	else if (__pmGetArchiveEnd(&logctl, &tv) < 0)
 	    printf("  ending     UNKNOWN\n");
 	else {
-	    ddmm = pmCtime((const time_t *)&tv.tv_sec, buffer);
+	    ddmm = pmCtime(&tv.tv_sec, buffer);
 	    ddmm[10] = '\0';
 	    yr = &ddmm[20];
 	    printf("  ending     %s ", ddmm);
@@ -410,7 +405,7 @@ main(int argc, char *argv[])
 	}
 	if (Lflag) {
 	    printf("Archive timezone: %s\n", golden.ill_tz);
-	    printf("PID for pmlogger: %d\n", (int)golden.ill_pid);
+	    printf("PID for pmlogger: %d\n", golden.ill_pid);
 	}
     }
 
