@@ -14,10 +14,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include <ctype.h>
@@ -107,7 +103,7 @@ addpmns(__pmnsNode *base, char *name, __pmnsNode *p)
     char	*tail;
     ptrdiff_t	nch;
     __pmnsNode	*np;
-    __pmnsNode	*lastp;
+    __pmnsNode	*lastp = NULL;
 
     for (tail = name; *tail && *tail != '.'; tail++)
 	;
@@ -192,13 +188,13 @@ main(int argc, char **argv)
     int		dupok = 0;
     int		errflag = 0;
     __pmnsNode	*tmp;
-    char	cmd[1024];
+    char	cmd[MAXPATHLEN + 64];
     char	pmnscomp[MAXPATHLEN];
-    
+
     umask((mode_t)022);		/* anything else is pretty silly */
     __pmSetProgname(argv[0]);
-    snprintf(pmnscomp, sizeof(pmnscomp), "%s/pmnscomp",
-		pmGetConfig("PCP_BINADM_DIR"));
+    snprintf(pmnscomp, sizeof(pmnscomp), "%s%c" "pmnscomp",
+		pmGetConfig("PCP_BINADM_DIR"), __pmPathSeparator());
 
     while ((c = getopt(argc, argv, "aD:dfv?")) != EOF) {
 	switch (c) {
@@ -263,14 +259,12 @@ Options:\n\
      * from here on, ignore SIGHUP, SIGINT and SIGTERM to protect
      * the integrity of the new ouput file
      */
-#if defined(HAVE_SIGHUP)
-    signal(SIGHUP, SIG_IGN);
-#endif
-    signal(SIGINT, SIG_IGN);
-    signal(SIGTERM, SIG_IGN);
+    __pmSetSignalHandler(SIGHUP, SIG_IGN);
+    __pmSetSignalHandler(SIGINT, SIG_IGN);
+    __pmSetSignalHandler(SIGTERM, SIG_IGN);
 
     if ((outf = fopen(argv[argc-1], "w+")) == NULL) {
-	fprintf(stderr, "%s: Error: cannot create output PMNS file \"%s\": %s\n", pmProgname, argv[argc-1], strerror(oserror()));
+	fprintf(stderr, "%s: Error: cannot create output PMNS file \"%s\": %s\n", pmProgname, argv[argc-1], osstrerror());
 	exit(1);
     }
 
